/************************************************************************
 * Lesson22Dbo performs the database input and output for our project.
 * Each instance of this class represents a single row in table
 * PUBLIC.LESSON22.
 * 
 * The class contains static methods for row retrieval and instance
 * methods for inserts and updates.
 * It also has a static method for batch delete, for illustration
 * purposes.
 */
package org.hardknockjava.lesson22.exercise01;

import java.io.InputStream;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

public class Lesson22Exercise01 {
    private static Connection conn = null;
    private static final String CATALOG = null;
    private static final String SCHEMA = "public";

    public static void main(String[] args) throws Exception {
        connect();
        DatabaseMetaData meta = conn.getMetaData();

        // Get the names of the tables in catalog null, schema "public".
        List<String> tableList = getTables(meta, CATALOG, SCHEMA);
        
        // List the tables and their columns.
        for (String tableName : tableList) {
            listColumns(meta, CATALOG, SCHEMA, tableName);
            System.out.println();
        }

        close();
    }

    // Establish the connection and create PreparedStatements.
    public static void connect() throws Exception {
        // Get the URL, user ID, and password from the properties file.
        InputStream is = Lesson22Exercise01.class.getClassLoader().getResourceAsStream("lesson22.properties");
        Properties props = new Properties();
        props.load(is);
        String url = props.getProperty("url");
        String user = props.getProperty("user");
        String password = props.getProperty("password");
        conn = DriverManager.getConnection(url, user, password);
        conn.setAutoCommit(false);
    }

    private static List<String> getTables(DatabaseMetaData meta, String catalog, String schema) throws SQLException {
        List<String> tableList = new ArrayList<String>();
        ResultSet rs = meta.getTables(catalog, schema, "%", null);
        while (rs.next()) {
            tableList.add(rs.getString("TABLE_NAME"));
        }
        rs.close();
        return tableList;
    }

    private static void listColumns(DatabaseMetaData meta, String catalog, String schema, String tableName) throws SQLException {
        ResultSet rs = meta.getColumns(catalog, schema, tableName, "%");
        System.out.println("Table: " + tableName);
        while (rs.next()) {
            String columnName = rs.getString("COLUMN_NAME");
            String columnType = rs.getString("TYPE_NAME");
            Integer columnSize = rs.getInt("COLUMN_SIZE");
            System.out.println("   " + columnName + " " + columnType + "  " + columnSize);
        }
        
    }

    // Close the PreparedStatements and the Connection.
    public static void close() throws SQLException {
        conn.close();
    }
}